@@ -59,7 +59,7 @@ extern double atan2();
 short savedir = 0;      /* for rhomboid/manhattan editing */
 char extchar[20];
 double saveratio;
-unsigned char texttype;
+u_char texttype;
 
 /*--------------------------------------*/
 /* Element constructor functions	*/
@@ -398,31 +398,6 @@ void drawdot(int xpos, int ypos)
    incr_changes(topobject);
 }
 
-/*----------------------*/
-/* Start spline mode.   */
-/*----------------------*/
-
-void startspline(xcWidget button, caddr_t clientdata, caddr_t calldata)
-{
-   if (eventmode == NORMAL_MODE) {
-      Wprintf("Click button1 and drag for spline.");
-      eventmode = SPLINE0_MODE;
-   }
-}
-
-/*----------------------*/
-/* Start label mode.    */
-/*----------------------*/
-
-void starttext(xcWidget button, pointertype mode, caddr_t calldata)
-{
-   if (eventmode == NORMAL_MODE) {
-      Wprintf("Click button1 for text.");
-      texttype = (u_char)mode;
-      eventmode = TEXT1_MODE;
-   }
-}
-
 /*--------------------------------------*/
 /* Sane default values for a label	*/
 /*--------------------------------------*/
@@ -457,7 +432,7 @@ void labeldefaults(labelptr newlabel, u_char dopin, int x, int y)
 /* Button handler when creating a label */
 /*--------------------------------------*/
 
-void textbutton(u_char dopin, XButtonEvent *event)
+void textbutton(u_char dopin, int x, int y)
 {
    labelptr *newlabel;
    XPoint userpt;
@@ -471,7 +446,7 @@ void textbutton(u_char dopin, XButtonEvent *event)
 
    NEW_LABEL(newlabel, topobject);
    areastruct.editpart = topobject->parts;
-   snap(event->x, event->y, &userpt);
+   snap(x, y, &userpt);
    labeldefaults(*newlabel, dopin, userpt.x, userpt.y);
 
    tmpheight = (short)(TEXTHEIGHT * (*newlabel)->scale);
@@ -524,7 +499,7 @@ labelptr findlabelcopy(labelptr curlabel, stringpart *curstring)
    for (tgen = topobject->plist; tgen < topobject->plist + topobject->parts; tgen++) {
       if (IS_LABEL(*tgen)) {
          tlab = TOLABEL(tgen);
-	 if (tlab->pin == False) continue;
+	 if (tlab->pin != LOCAL) continue;
 	 else if (tlab == curlabel) continue;  /* Don't count self! */
          else if (!stringcomp(tlab->string, curstring)) return tlab;  
       }
@@ -536,14 +511,17 @@ labelptr findlabelcopy(labelptr curlabel, stringpart *curstring)
 /* Interpret string and add to current label.			*/
 /* 	keypressed is a KeySym					*/
 /*	clientdata can pass information for label controls	*/
+/*								*/
+/* Return TRUE if labeltext handled the character, FALSE if the	*/
+/* character was not recognized.				*/
 /*--------------------------------------------------------------*/
 
-void labeltext(int keypressed, char *clientdata)
+Boolean labeltext(int keypressed, char *clientdata)
 {
    labelptr curlabel;
    stringpart *curpos, *labelbuf;
    int locpos;
-   Boolean do_redraw = False;
+   Boolean r, do_redraw = False;
    short tmplength, tmpheight, cfont;
    TextExtents tmpext;
 
@@ -551,7 +529,7 @@ void labeltext(int keypressed, char *clientdata)
 
    if (curlabel == NULL || textpos <= 0) {
       Wprintf("Error:  Bad label string");
-      return;
+      return FALSE;
    }
 
    /* find text segment of the current position */
@@ -559,41 +537,60 @@ void labeltext(int keypressed, char *clientdata)
 
    UDrawTLine(curlabel);
 
-   if (isbound(keypressed, XCF_Text_Delete) && textpos > 1) {
-      int curloc, strpos;
-      stringpart *strptr;
+   if (r = isbound(keypressed, XCF_Text_Delete)) {
+      if (textpos > 1) {
+         int curloc, strpos;
+         stringpart *strptr;
 
-      if (textend == 0) textend = textpos - 1;
+         if (textend == 0) textend = textpos - 1;
 
-      undrawtext(curlabel);
-      for (strpos = textpos - 1; strpos >= textend; strpos--) {
-	 strptr = findstringpart(strpos, &curloc, curlabel->string,
+         undrawtext(curlabel);
+         for (strpos = textpos - 1; strpos >= textend; strpos--) {
+	    strptr = findstringpart(strpos, &curloc, curlabel->string,
 			 areastruct.topinstance);
-	 if (curloc >= 0) {
-	    memmove(strptr->data.string + curloc,
+	    if (curloc >= 0) {
+	       memmove(strptr->data.string + curloc,
 			strptr->data.string + curloc + 1,
 			strlen(strptr->data.string + curloc + 1) + 1);
-	    if (strlen(strptr->data.string) == 0)
-	       deletestring(strptr, &curlabel->string, areastruct.topinstance);
-	 }
+	       if (strlen(strptr->data.string) == 0)
+	          deletestring(strptr, &curlabel->string, areastruct.topinstance);
+	    }
 
-         /* Don't delete any parameter boundaries---must use	*/
-         /* "unparameterize" command for that.			*/
+            /* Don't delete any parameter boundaries---must use	*/
+            /* "unparameterize" command for that.		*/
 
-	 else if (strptr != NULL) {
-	    if ((strptr->type != PARAM_START) && (strptr->type != PARAM_END))
-	       deletestring(strptr, &curlabel->string, areastruct.topinstance);
+	    else if (strptr != NULL) {
+	       if ((strptr->type != PARAM_START) && (strptr->type != PARAM_END))
+	          deletestring(strptr, &curlabel->string, areastruct.topinstance);
+	       else
+	          textpos++;
+	    }
 	    else
-	       textpos++;
-	 }
-	 else
-	    Fprintf(stdout, "Error:  Unexpected NULL string part\n");
-         textpos--;
+	       Fprintf(stdout, "Error:  Unexpected NULL string part\n");
+            textpos--;
+         }
+         textend = 0;
+         do_redraw = True;
       }
-      textend = 0;
-      do_redraw = True;
    }
-   else if (isbound(keypressed, XCF_Text_Return)) {
+   else if (r = isbound(keypressed, XCF_Text_Delete_Param)) {
+      if (textpos > 1) {
+         int curloc, strpos;
+         stringpart *strptr;
+
+         strptr = findstringpart(textpos - 1, &curloc, curlabel->string,
+			 areastruct.topinstance);
+         if ((curloc < 0) && (strptr != NULL) && (strptr->type == PARAM_END)) {
+	    undrawtext(curlabel);
+	    while (strptr->type != PARAM_START)
+	       strptr = findstringpart(--strpos, &curloc, curlabel->string,
+			areastruct.topinstance);
+	    unmakeparam(curlabel, strptr);
+            do_redraw = True;
+	 }
+      }
+   }
+   else if (r = isbound(keypressed, XCF_Text_Return)) {
       Boolean hasstuff = False;   /* Check for null string */
       stringpart *tmppos;
 
@@ -601,11 +598,11 @@ void labeltext(int keypressed, char *clientdata)
 	 if (tmppos->type == PARAM_START) hasstuff = True;
 	 else if (tmppos->type == TEXT_STRING) hasstuff = True;
       }
-      XDefineCursor(dpy, areastruct.areawin, CROSS);
+      XDefineCursor(dpy, areastruct.areawin, DEFAULTCURSOR);
 
       Wprintf("");
 
-      if (hasstuff && (eventmode != TEXT3_MODE && eventmode != CATTEXT_MODE)) {
+      if (hasstuff && (eventmode != ETEXT_MODE && eventmode != CATTEXT_MODE)) {
 	 topobject->parts++;
          register_for_undo(XCF_Text, UNDO_DONE, areastruct.topinstance,
                     curlabel);
@@ -614,7 +611,7 @@ void labeltext(int keypressed, char *clientdata)
 	 invalidate_netlist(topobject);
 
       }
-      else if (!hasstuff && (eventmode == TEXT3_MODE)) {
+      else if (!hasstuff && (eventmode == ETEXT_MODE)) {
 	 if (areastruct.editpart < topobject->parts) {
 	    short *sptr = allocselect();
 	    *sptr = areastruct.editpart;
@@ -708,13 +705,15 @@ void labeltext(int keypressed, char *clientdata)
       else
          calcbbox(areastruct.topinstance);
 
-      return;
+      return r;
    }
-   else if (isbound(keypressed, XCF_Text_Right) && curpos != NULL)
-      textpos++;
-   else if (isbound(keypressed, XCF_Text_Left) && textpos > 1)
-      textpos--;
-   else if (isbound(keypressed, XCF_Text_Down)) {
+   else if (r = isbound(keypressed, XCF_Text_Right)) {
+      if (curpos != NULL) textpos++;
+   }
+   else if (r = isbound(keypressed, XCF_Text_Left)) {
+      if (textpos > 1) textpos--;
+   }
+   else if (r = isbound(keypressed, XCF_Text_Down)) {
       while (curpos != NULL) {
 	 textpos++;
 	 curpos = findstringpart(textpos, &locpos, curlabel->string,
@@ -724,7 +723,7 @@ void labeltext(int keypressed, char *clientdata)
 	       break;
       }
    }
-   else if (isbound(keypressed, XCF_Text_Up)) {
+   else if (r = isbound(keypressed, XCF_Text_Up)) {
       while (textpos > 1) {
 	 textpos--;
 	 curpos = findstringpart(textpos, &locpos, curlabel->string,
@@ -735,11 +734,11 @@ void labeltext(int keypressed, char *clientdata)
 	 }
       }
    }
-   else if (isbound(keypressed, XCF_Text_Home))
+   else if (r = isbound(keypressed, XCF_Text_Home))
       textpos = 1;
-   else if (isbound(keypressed, XCF_Text_End))
+   else if (r = isbound(keypressed, XCF_Text_End))
       textpos = stringlength(curlabel->string, True, areastruct.topinstance);
-   else if (isbound(keypressed, XCF_Text_Split)) {
+   else if (r = isbound(keypressed, XCF_Text_Split)) {
       labelptr *newlabel;
       XPoint points[4], points1[4], points2[4];
 
@@ -824,8 +823,10 @@ void labeltext(int keypressed, char *clientdata)
 	 Wprintf("Error in insertion.  Ignoring.");
 	 deletestring(newpart, &curlabel->string, areastruct.topinstance);
       }
-      else
+      else {
          textpos++;
+	 r = TRUE;
+      }
       do_redraw = True;
    }
 
@@ -842,7 +843,8 @@ void labeltext(int keypressed, char *clientdata)
       if (locpos < 0) {
 
          /* Find part of string which is immediately in front of textpos */
-         lastpos = findstringpart(textpos - 1, &locpos, curlabel->string, areastruct.topinstance);
+         lastpos = findstringpart(textpos - 1, &locpos, curlabel->string,
+		areastruct.topinstance);
 
 	 /* No text on either side to attach to: make a new text segment */
 	 if (locpos < 0) {
@@ -869,6 +871,7 @@ void labeltext(int keypressed, char *clientdata)
       }
       textpos++;	/* move forward to next text position */
       do_redraw = True;
+      r = TRUE;
    }
 
    /* Redraw the label */
@@ -876,10 +879,12 @@ void labeltext(int keypressed, char *clientdata)
    if (do_redraw) {
       short beglength;
 
-      tmpext = ULength(curlabel->string, areastruct.topinstance, curlabel->scale, textpos, NULL);
+      tmpext = ULength(curlabel->string, areastruct.topinstance, curlabel->scale,
+		textpos, NULL);
       beglength = tmpext.width;
 	
-      tmpext = ULength(curlabel->string, areastruct.topinstance, curlabel->scale, 0, NULL);
+      tmpext = ULength(curlabel->string, areastruct.topinstance, curlabel->scale,
+		0, NULL);
       tmplength = tmpext.width;
       tmpheight = (short)(curlabel->scale * TEXTHEIGHT);
       areastruct.origin.x = curlabel->position.x + (curlabel->justify & NOTLEFT
@@ -894,23 +899,27 @@ void labeltext(int keypressed, char *clientdata)
       XcSetFunction(GXcopy);
       redrawtext(curlabel);
    }
-   UDrawTLine(curlabel);
 
-   /* Report on characters at the cursor position in the message window */
+   if (r || do_redraw) {
+      UDrawTLine(curlabel);
 
-   charreport(curlabel);
+      /* Report on characters at the cursor position in the message window */
 
-   /* find current font and adjust menubuttons as necessary */
+      charreport(curlabel);
 
-   cfont = findcurfont(textpos, curlabel->string, areastruct.topinstance);
-   if (cfont < 0) {
-      Wprintf("Error:  Illegal label string");
-      return;
+      /* find current font and adjust menubuttons as necessary */
+
+      cfont = findcurfont(textpos, curlabel->string, areastruct.topinstance);
+      if (cfont < 0) {
+         Wprintf("Error:  Illegal label string");
+         return r;
+      }
+      else
+         setfontmarks(cfont, -1);
+
+      textend = 0;
    }
-   else
-      setfontmarks(cfont, -1);
-
-   textend = 0;
+   return r;
 }
 
 /*-------------------------------------*/
@@ -937,7 +946,7 @@ void rejustify(short mode)
    Boolean changed = False;
    static short transjust[] = {15, 13, 12, 7, 5, 4, 3, 1, 0};
 
-   if (eventmode == TEXT2_MODE || eventmode == TEXT3_MODE) {
+   if (eventmode == TEXT_MODE || eventmode == ETEXT_MODE) {
       curlabel = TOLABEL(EDITPART);
       UDrawTLine(curlabel);
       undrawtext(curlabel);
@@ -960,10 +969,12 @@ void rejustify(short mode)
       	    curlabel->justify = transjust[mode] |
 		(curlabel->justify & NONJUSTFIELD);
             if (jsave != curlabel->justify) changed = True;
-	    redrawtext(curlabel);
 	 }
       }
-      if (eventmode != PRESS_MODE && eventmode != COPY2_MODE) unselect_all();
+      if (eventmode != MOVE_MODE && eventmode != COPY_MODE)
+	 unselect_all();
+      else
+	 draw_all_selected();
    }
    if (curlabel == NULL)
       Wprintf("No labels chosen to rejustify");
@@ -999,7 +1010,7 @@ void splinedefaults(splineptr newspline, int x, int y)
 /* Start drawing a spline. */
 /*-------------------------*/
 
-void splinebutton(XButtonEvent *event)
+void splinebutton(int x, int y)
 {
    splineptr *newspline;
    XPoint userpt;
@@ -1007,7 +1018,7 @@ void splinebutton(XButtonEvent *event)
    NEW_SPLINE(newspline, topobject);
    areastruct.editpart = topobject->parts;
 
-   snap(event->x, event->y, &userpt);
+   snap(x, y, &userpt);
    areastruct.editcycle = 3;
    splinedefaults(*newspline, userpt.x, userpt.y);
 
@@ -1057,30 +1068,6 @@ void trackspline(xcWidget w, caddr_t clientdata, caddr_t calldata)
    flusharea();
 }
 
-/*--------------------------*/
-/* Start drawing a polygon. */
-/*--------------------------*/
-
-void startpoly(xcWidget button, caddr_t clientdata, caddr_t calldata)
-{
-   if (eventmode == NORMAL_MODE) {
-      Wprintf("Click button1 to start the polygon.");
-      eventmode = POLY_MODE;
-   }
-}
-
-/*-----------------------*/
-/* Start drawing an arc. */
-/*-----------------------*/
-
-void startarc(xcWidget button, caddr_t clientdata, caddr_t calldata)
-{
-   if (eventmode == NORMAL_MODE) {
-      Wprintf("Click button1 and drag to make arc.");
-      eventmode = ARC0_MODE;
-   }
-}
-
 /*--------------------------------------*/
 /* Set default values for an arc	*/
 /*--------------------------------------*/
@@ -1104,14 +1091,14 @@ void arcdefaults(arcptr newarc, int x, int y)
 /* Button handler when creating an arc */
 /*-------------------------------------*/
 
-void arcbutton(XButtonEvent *event)
+void arcbutton(int x, int y)
 {
    arcptr *newarc;
    XPoint userpt;
 
    NEW_ARC(newarc, topobject);
    areastruct.editpart = topobject->parts;
-   snap(event->x, event->y, &userpt);
+   snap(x, y, &userpt);
    areastruct.editcycle = 0;
    saveratio = 1.0;
    arcdefaults(*newarc, userpt.x, userpt.y);
@@ -1196,18 +1183,6 @@ void trackarc(xcWidget w, caddr_t clientdata, caddr_t calldata)
    flusharea();
 }
 
-/*----------------------*/
-/* Start drawing a box. */
-/*----------------------*/
-
-void startbox(xcWidget button, caddr_t clientdata, caddr_t calldata)
-{
-   if (eventmode == NORMAL_MODE) {
-      Wprintf("Click button1 and drag to make box.");
-      eventmode = BOX0_MODE;
-   }
-}
-
 /*--------------------------------------*/
 /* Sane default values for a polygon	*/
 /*--------------------------------------*/
@@ -1238,14 +1213,14 @@ void polydefaults(polyptr newpoly, int number, int x, int y)
 /* Button handler when creating a box */
 /*------------------------------------*/
 
-void boxbutton(XButtonEvent *event)
+void boxbutton(int x, int y)
 {
    polyptr *newbox;
    XPoint userpt;
 
    NEW_POLY(newbox, topobject);
    areastruct.editpart = topobject->parts;
-   snap(event->x, event->y, &userpt);
+   snap(x, y, &userpt);
    polydefaults(*newbox, 4, userpt.x, userpt.y);
 
    XcSetXORFg(areastruct.color, BACKGROUND);
@@ -1957,12 +1932,12 @@ void poly_add_point(polyptr thispoly, XPoint *newpoint) {
 /* ButtonPress handler while a wire is being drawn */
 /*-------------------------------------------------*/
 
-void wirebutton(XButtonEvent *event)
+void wire_op(int op, int x, int y)
 {
    XPoint userpt, *tpoint;
    polyptr newwire;
 
-   snap(event->x, event->y, &userpt);
+   snap(x, y, &userpt);
 
    newwire = TOPOLY(EDITPART);
    if (areastruct.manhatn) manhattanize(&userpt, newwire);
@@ -1975,8 +1950,16 @@ void wirebutton(XButtonEvent *event)
    tpoint->x = userpt.x;
    tpoint->y = userpt.y;
 
+   /* cancel wire operation completely */
+   if (op == XCF_Cancel) {
+      free(newwire->points);
+      free(newwire);
+      newwire = NULL;
+      eventmode = NORMAL_MODE;
+   }
+
    /* back up one point; prevent length zero wires */
-   if ((event->button == Button3) || ((tpoint - 1)->x == userpt.x &&
+   else if ((op == XCF_Cancel_Last) || ((tpoint - 1)->x == userpt.x &&
 	   (tpoint - 1)->y == userpt.y)) {
       if (newwire->number <= 2) {
 	 free(newwire->points);
@@ -1990,17 +1973,17 @@ void wirebutton(XButtonEvent *event)
       }
    }
 
-   if (newwire && event->button == Button1) {
+   if (newwire && (op == XCF_Wire || op == XCF_Continue_Element)) {
       if (newwire->number == 2)
 	 newwire->style = areastruct.style;
       poly_add_point(newwire, &userpt);
    }
-   else if (newwire == NULL || event->button != Button3)
+   else if ((newwire == NULL) || op == XCF_Finish_Element || op == XCF_Cancel)
       xcRemoveEventHandler(areastruct.area, PointerMotionMask, False,
          (xcEventHandler)trackwire, NULL);
 
    if (newwire) {
-      if (event->button == Button2) {
+      if (op == XCF_Finish_Element) {
          XcSetFunction(GXcopy);
          XcSetForeground(newwire->color);
 	 topobject->parts++;
@@ -2009,9 +1992,14 @@ void wirebutton(XButtonEvent *event)
 	 register_for_undo(XCF_Wire, UNDO_DONE, areastruct.topinstance, newwire);
       }
       UDrawPolygon(newwire);
-      if (event->button == Button3)
+      if (op == XCF_Cancel_Last)
 	 checkwarp(newwire->points + newwire->number - 1);
    }
+
+   if (op == XCF_Finish_Element) {
+      eventmode = NORMAL_MODE;
+      singlebbox(EDITPART);
+   }
 }
 
 /*-------------------------------------------------------------------------*/
