@@ -202,13 +202,6 @@ void setgrid(xcWidget w, float *dataptr)
    /* Might be nice in the future to make it accept any input. . .    */
 
    switch (xobjs.pagelist[areastruct.page]->coordstyle) {
-      case INTERNAL:
-	 if (sscanf(_STR2, "%f", &fval) == 0) {
-	    *dataptr = oldvalue;
-	    Wprintf("Illegal value");
-	 }
-	 else *dataptr = fval / iscale;
-	 break;
       case CM:
          oscale = xobjs.pagelist[areastruct.page]->outscale * CMSCALE;
 	 if (sscanf(_STR2, "%f", &fval) == 0) {
@@ -251,9 +244,6 @@ void measurestr(float value, char *buffer)
         (float)(xobjs.pagelist[areastruct.page]->drawingscale.x);
 
    switch (xobjs.pagelist[areastruct.page]->coordstyle) {
-      case INTERNAL:
-         sprintf(buffer, "%5.3f", value * iscale);
-	 break;
       case CM:
          oscale = xobjs.pagelist[areastruct.page]->outscale * CMSCALE;
          sprintf(buffer, "%5.3f cm", value * iscale * oscale / IN_CM_CONVERT);
@@ -353,7 +343,7 @@ void settsize(xcWidget w, labelptr settext)
 
    /* In edit mode, add font scale change. */
 
-   if (eventmode == TEXT2_MODE || eventmode == TEXT3_MODE) {
+   if (eventmode == TEXT_MODE || eventmode == ETEXT_MODE) {
       if (textpos > 0 || textpos < stringlength(settext->string, True, areastruct.topinstance)) {
 	 undrawtext(settext);
 	 strptr = findstringpart(textpos - 1, NULL, settext->string, areastruct.topinstance);
@@ -665,7 +655,7 @@ void getkern(xcWidget button, caddr_t nulldata, caddr_t calldata)
 
    strcpy(buffer, "0,0");
 
-   if (eventmode == TEXT2_MODE || eventmode == TEXT3_MODE) {
+   if (eventmode == TEXT_MODE || eventmode == ETEXT_MODE) {
       labelptr curlabel = TOLABEL(EDITPART);
       strptr = findstringpart(textpos - 1, NULL, curlabel->string, areastruct.topinstance);
       nextptr = findstringpart(textpos, NULL, curlabel->string, areastruct.topinstance);
@@ -765,7 +755,7 @@ labelptr gettextsize(float **floatptr)
 
    if (floatptr) *floatptr = &areastruct.textscale;
 
-   if (eventmode == TEXT2_MODE || eventmode == TEXT3_MODE) {
+   if (eventmode == TEXT_MODE || eventmode == ETEXT_MODE) {
       if (textpos > 0 || textpos < stringlength(settext->string, True,
 		areastruct.topinstance)) {
          settext = *((labelptr *)EDITPART);
@@ -1061,6 +1051,66 @@ void setpagelabel(xcWidget w, char *dataptr)
    renamepage(areastruct.page);
 }
 
+/*--------------------------------------------------------------*/
+/* Change the Button1 binding for a particular mode (non-Tcl)	*/
+/*--------------------------------------------------------------*/
+
+#ifndef TCL_WRAPPER
+
+/*--------------------------------------------------------------*/
+/* Change to indicated tool (key binding w/o value) */
+/*--------------------------------------------------------------*/
+
+void changetool(xcWidget w, pointertype value, caddr_t nulldata)
+{
+   mode_rebinding((int)value, -1);
+   highlightexcl(w, (int)value, -1);
+}
+
+/*--------------------------------------------------------------*/
+/* Execute function binding for the indicated tool if something	*/
+/* is selected;  otherwise, change to the indicated tool mode.	*/
+/*--------------------------------------------------------------*/
+
+void exec_or_changetool(xcWidget w, pointertype value, caddr_t nulldata)
+{
+   if (areastruct.selects > 0)
+      mode_tempbinding((int)value, -1);
+   else {
+      mode_rebinding((int)value, -1);
+      highlightexcl(w, (int)value, -1);
+   }
+}
+
+/*--------------------------------------------------------------*/
+/* Special case of exec_or_changetool(), where an extra value	*/
+/* is passed to the routine indication amount of rotation, or	*/
+/* type of flip operation.					*/
+/*--------------------------------------------------------------*/
+
+void rotatetool(xcWidget w, pointertype value, caddr_t nulldata)
+{
+   if (areastruct.selects > 0)
+      mode_tempbinding(XCF_Rotate, (int)value);
+   else {
+      mode_rebinding(XCF_Rotate, (int)value);
+      highlightexcl(w, (int)XCF_Rotate, (int)value);
+   }
+}
+
+/*--------------------------------------------------------------*/
+/* Special case of changetool() for pan mode, where a value is	*/
+/* required to pass to the key binding routine.			*/
+/*--------------------------------------------------------------*/
+
+void pantool(xcWidget w, pointertype value, caddr_t nulldata)
+{
+   mode_rebinding(XCF_Pan, (int)value);
+   highlightexcl(w, (int)XCF_Pan, (int)value);
+}
+
+#endif  /* !TCL_WRAPPER */
+
 /*--------------------------------------------------------------*/
 /* Add a new font name to the list of known fonts		*/
 /* Register the font number for the Alt-F cycling mechanism	*/
@@ -1212,6 +1262,124 @@ void toggleexcl(xcWidget widget, menuptr menu, int menulength)
    XtSetValues(widget, args, 1);
 }
 
+/*----------------------------------------------------------------------*/
+/* Cursor changes based on toolbar mode					*/
+/*----------------------------------------------------------------------*/
+
+void toolcursor(int mode)
+{
+   /* Some cursor types for different modes */
+   switch(mode) {
+      case XCF_Spline:
+      case XCF_Move:
+      case XCF_Join:
+      case XCF_Unjoin:
+	 XDefineCursor (dpy, areastruct.areawin, ARROW);
+	 areastruct.defaultcursor = &ARROW;
+	 break;
+      case XCF_Pan:
+	 XDefineCursor (dpy, areastruct.areawin, HAND);
+	 areastruct.defaultcursor = &HAND;
+	 break;
+      case XCF_Delete:
+	 XDefineCursor (dpy, areastruct.areawin, SCISSORS);
+	 areastruct.defaultcursor = &SCISSORS;
+	 break;
+      case XCF_Copy:
+	 XDefineCursor (dpy, areastruct.areawin, COPYCURSOR);
+	 areastruct.defaultcursor = &COPYCURSOR;
+	 break;
+      case XCF_Push:
+      case XCF_Select_Save:
+	 XDefineCursor (dpy, areastruct.areawin, QUESTION);
+	 areastruct.defaultcursor = &QUESTION;
+	 break;
+      case XCF_Rotate:
+      case XCF_Flip_X:
+      case XCF_Flip_Y:
+	 XDefineCursor (dpy, areastruct.areawin, ROTATECURSOR);
+	 areastruct.defaultcursor = &ROTATECURSOR;
+	 break;
+      case XCF_Edit:
+	 XDefineCursor (dpy, areastruct.areawin, EDCURSOR);
+	 areastruct.defaultcursor = &EDCURSOR;
+	 break;
+      case XCF_Arc:
+	 XDefineCursor (dpy, areastruct.areawin, CIRCLE);
+	 areastruct.defaultcursor = &CIRCLE;
+	 break;
+      case XCF_Text:
+      case XCF_Pin_Label:
+      case XCF_Pin_Global:
+      case XCF_Info_Label:
+	 XDefineCursor (dpy, areastruct.areawin, TEXTPTR);
+	 areastruct.defaultcursor = &TEXTPTR;
+	 break;
+      default:
+	 XDefineCursor (dpy, areastruct.areawin, CROSS);
+	 areastruct.defaultcursor = &CROSS;
+	 break;
+   }
+}
+
+/*--------------------------------------------------------------------------*/
+/* Highlight one of a set of toolbar items, only one of which can be active */
+/*--------------------------------------------------------------------------*/
+
+void highlightexcl(xcWidget widget, int func, int value)
+{
+   Arg          args[2];
+   xcWidget     parent = xcParent(PanToolButton);
+   xcWidget     sibling, self = (xcWidget)NULL;
+   toolbarptr	titem, ttest;
+
+   /* remove highlight from all widgets in the toolbar */
+
+   XtSetArg(args[0], XtNbackground, BACKGROUND);
+   XtSetArg(args[1], XtNborderColor, SNAPCOLOR);
+   for (titem = ToolBar; titem < ToolBar + toolbuttons; titem++) {
+      sibling = XtNameToWidget(parent, titem->name);
+      if (sibling == widget)
+	 self = sibling;
+      else
+         XtSetValues(sibling, args, 2);
+   }
+
+   if (self == (xcWidget)NULL) {
+      /* We invoked a toolbar button from somewhere else.  Highlight	*/
+      /* the toolbar associated with the function value.		*/
+      switch (func) {
+	 case XCF_Pan:
+	    self = PanToolButton;
+	    break;
+
+	 case XCF_Rotate:
+	    if (value > 0)
+	       self = RotPToolButton;
+	    else
+	       self = RotNToolButton;
+	    break;
+
+	 default:
+	    for (titem = ToolBar; titem < ToolBar + toolbuttons; titem++) {
+	       if (func == (int)titem->passeddata) {
+                  self = XtNameToWidget(parent, titem->name);
+	          break;
+	       }
+	    }
+	    break;
+      }
+   }
+
+   /* Now highlight the currently pushed widget */
+
+   if (self != (xcWidget)NULL) {
+      XtSetArg(args[0], XtNbackground, RATSNESTCOLOR);
+      XtSetArg(args[1], XtNborderColor, RATSNESTCOLOR);
+      XtSetValues(self, args, 2);
+   }
+}
+
 #endif
 
 /*--------------------*/
@@ -1287,7 +1455,8 @@ void inversecolor(xcWidget w, Boolean *boolvalue, caddr_t calldata)
 
    if (w == NULL) w = OptionsAltColorsButton;
    if (w != NULL) toggle(w, boolvalue, calldata);
-   if (eventmode == NORMAL_MODE) XDefineCursor (dpy, areastruct.areawin, CROSS);
+   if (eventmode == NORMAL_MODE)
+      XDefineCursor (dpy, areastruct.areawin, DEFAULTCURSOR);
 }
 
 #endif
@@ -1322,7 +1491,6 @@ void togglegrid(u_short type)
    if (type == CM) button = XtNameToWidget(bparent, "Centimeters");
    else if (type == FRAC_INCH) button = XtNameToWidget(bparent, "Fractional Inches");
    else if (type == DEC_INCH) button = XtNameToWidget(bparent, "Decimal Inches");
-   else if (type == INTERNAL) button = XtNameToWidget(bparent, "Internal Units");
    else button = XtNameToWidget(bparent, "Coordinates");
    toggleexcl(button, GridStyles, XtNumber(GridStyles));
    W1printf(" ");
@@ -1374,7 +1542,7 @@ void getgridtype(xcWidget button, pointertype value, caddr_t calldata)
    xobjs.pagelist[areastruct.page]->coordstyle = (short)value;
 
    switch(value) {
-      case FRAC_INCH: case DEC_INCH: case INTERNAL:
+      case FRAC_INCH: case DEC_INCH:
 	 if (oldtype == CM) {
             xobjs.pagelist[areastruct.page]->outscale *= scalefac;
 #ifndef TCL_WRAPPER
@@ -1435,10 +1603,7 @@ int findemptylib()
 
 /*----------------------------------------------*/
 /* Make new library and add a new button to the */
-/* "Libraries" cascaded menu.	If "force" is	*/
-/* FALSE, then return the number of the first	*/
-/* empty library found.  If "force" is TRUE,	*/
-/* then always create a new library.		*/ 
+/* "Libraries" cascaded menu.			*/
 /*----------------------------------------------*/
 
 int createlibrary(Boolean force)
@@ -2034,7 +2199,7 @@ void setcolor(xcWidget w, pointertype value, caddr_t calldata)
 
 #endif
    
-   if (eventmode == TEXT2_MODE || eventmode == TEXT3_MODE) {
+   if (eventmode == TEXT_MODE || eventmode == ETEXT_MODE) {
       labelptr curlabel = TOLABEL(EDITPART);
       strptr = findstringpart(textpos - 1, NULL, curlabel->string, areastruct.topinstance);
       nextptr = findstringpart(textpos, NULL, curlabel->string, areastruct.topinstance);
@@ -2066,7 +2231,7 @@ void setcolor(xcWidget w, pointertype value, caddr_t calldata)
 
    setcolormark(cval);
    if (!selected) {
-      if (eventmode != TEXT2_MODE && eventmode != TEXT3_MODE)
+      if (eventmode != TEXT_MODE && eventmode != ETEXT_MODE)
          areastruct.color = cval;      
       overdrawpixmap(w);
    }
@@ -2322,15 +2487,13 @@ void stringparam(xcWidget w, caddr_t clientdata, caddr_t calldata)
 {
    genericptr *settext;
 
-      if (eventmode == TEXT2_MODE || eventmode == TEXT3_MODE) {
+      if (eventmode == TEXT_MODE || eventmode == ETEXT_MODE) {
          settext = (genericptr *)EDITPART;
          makeparam(TOLABEL(settext), _STR2);
          unselect_all();
 	 setparammarks(NULL);
       }
       else if (checkselect(LABEL)) parameterize(P_SUBSTRING, _STR2);
-      else
-         eventmode = LPARAM_MODE;
 }
 
 /*--------------------------------------------------------------*/
@@ -2345,13 +2508,8 @@ void startparam(xcWidget w, pointertype value, caddr_t calldata)
       strcpy(_STR2, (calldata != NULL) ? (char *)calldata : "substring");
       stringparam(w, NULL, NULL);
    }
-   else {
-      if ((eventmode == NORMAL_MODE) && (areastruct.selects == 0)) { 
-         eventmode = IPARAM_MODE;
-         Wprintf("Click on object to parameterize.");
-      }      
-      else parameterize((int)value, (char *)calldata);
-   }
+   else if ((eventmode != NORMAL_MODE) || (areastruct.selects > 0))
+	 parameterize((int)value, (char *)calldata);
 }
 
 /*---------------------------------------------------------------*/
@@ -2360,14 +2518,7 @@ void startparam(xcWidget w, pointertype value, caddr_t calldata)
 
 void startunparam(xcWidget w, pointertype value, caddr_t calldata)
 {
-   if (areastruct.selects == 0) {
-      if (value == LABEL)
-         eventmode = ULPARAM_MODE;
-      else
-         eventmode = UIPARAM_MODE;
-      Wprintf("Select element or substring to remove parameters.");
-   }
-   else
+   if (areastruct.selects > 0)
       unparameterize((int)value);
    unselect_all();
    setparammarks(NULL);
@@ -2481,7 +2632,7 @@ void setvjust(xcWidget w, pointertype value, caddr_t nulldata)
    labelptr settext;
    short labelcount = 0;
 
-   if (eventmode == TEXT2_MODE || eventmode == TEXT3_MODE) {
+   if (eventmode == TEXT_MODE || eventmode == ETEXT_MODE) {
       settext = *((labelptr *)EDITPART);
       setjust(w, value, settext, 2);
    }
@@ -2510,7 +2661,7 @@ void sethjust(xcWidget w, pointertype value, caddr_t nulldata)
    labelptr settext;
    short labelcount = 0;
 
-   if (eventmode == TEXT2_MODE || eventmode == TEXT3_MODE) {
+   if (eventmode == TEXT_MODE || eventmode == ETEXT_MODE) {
       settext = *((labelptr *)EDITPART);
       setjust(w, value, settext, 1);
    }
@@ -2539,7 +2690,7 @@ void setjustbit(xcWidget w, pointertype value, caddr_t nulldata)
    labelptr settext;
    short labelcount = 0;
 
-   if (eventmode == TEXT2_MODE || eventmode == TEXT3_MODE) {
+   if (eventmode == TEXT_MODE || eventmode == ETEXT_MODE) {
       settext = *((labelptr *)EDITPART);
       dojustifybit(w, settext, (short)value);
    }
@@ -2567,7 +2718,7 @@ void setpinjustbit(xcWidget w, pointertype value, caddr_t nulldata)
    short *fselect;
    labelptr settext;
 
-   if (eventmode == TEXT2_MODE || eventmode == TEXT3_MODE) {
+   if (eventmode == TEXT_MODE || eventmode == ETEXT_MODE) {
       settext = *((labelptr *)EDITPART);
       if (settext->pin)
 	 dopinjustbit(w, settext, (short)value);
@@ -2804,7 +2955,7 @@ void setfontval(xcWidget w, pointertype value, labelptr settext)
 
    if ((newfont = (int)findbestfont(tc, (short)value, -1, -1)) < 0) return;
 
-   if (eventmode == TEXT2_MODE || eventmode == TEXT3_MODE) {
+   if (eventmode == TEXT_MODE || eventmode == ETEXT_MODE) {
       sprintf(_STR, "Font is now %s", fonts[newfont].psname);
       sprintf(_STR2, "%d", newfont);
       labeltext(FONT_NAME, (char *)&newfont);
@@ -2829,9 +2980,9 @@ void setfont(xcWidget w, pointertype value, caddr_t calldata)
    short labelcount = 0;
 
    if (eventmode == CATALOG_MODE || eventmode == FONTCAT_MODE ||
-		eventmode == FONTCAT2_MODE) return;
+		eventmode == EFONTCAT_MODE) return;
 
-   if (eventmode == TEXT2_MODE || eventmode == TEXT3_MODE) {
+   if (eventmode == TEXT_MODE || eventmode == ETEXT_MODE) {
       settext = *((labelptr *)EDITPART);
       setfontval(w, value, settext);
       charreport(settext);
@@ -2896,7 +3047,7 @@ void setfontstyle(xcWidget w, pointertype value, labelptr settext)
 
    if ((newfont = (int)findbestfont(tc, -1, (short)value, -1)) < 0) return;
 
-   if (eventmode == TEXT2_MODE || eventmode == TEXT3_MODE) {
+   if (eventmode == TEXT_MODE || eventmode == ETEXT_MODE) {
       sprintf(_STR, "Font is now %s", fonts[newfont].psname);
       sprintf(_STR2, "%d", newfont);
       labeltext(FONT_NAME, (char *)&newfont);
@@ -2925,9 +3076,9 @@ void fontstyle(xcWidget w, pointertype value, caddr_t nulldata)
    short labelcount = 0;
 
    if (eventmode == CATALOG_MODE || eventmode == FONTCAT_MODE ||
-		eventmode == FONTCAT2_MODE) return;
+		eventmode == EFONTCAT_MODE) return;
 
-   if (eventmode == TEXT2_MODE || eventmode == TEXT3_MODE) {
+   if (eventmode == TEXT_MODE || eventmode == ETEXT_MODE) {
       settext = *((labelptr *)EDITPART);
       setfontstyle(w, value, settext);
       charreport(settext);
@@ -2992,7 +3143,7 @@ void setfontencoding(xcWidget w, pointertype value, labelptr settext)
 
    if ((newfont = (int)findbestfont(tc, -1, -1, (short)value)) < 0) return;
 
-   if (eventmode == TEXT2_MODE || eventmode == TEXT3_MODE) {
+   if (eventmode == TEXT_MODE || eventmode == ETEXT_MODE) {
       sprintf(_STR, "Font is now %s", fonts[newfont].psname);
       sprintf(_STR2, "%d", newfont);
       labeltext(FONT_NAME, (char *)&newfont);
@@ -3020,9 +3171,9 @@ void fontencoding(xcWidget w, pointertype value, caddr_t nulldata)
    short labelcount = 0;
 
    if (eventmode == CATALOG_MODE || eventmode == FONTCAT_MODE ||
-		eventmode == FONTCAT2_MODE) return;
+		eventmode == EFONTCAT_MODE) return;
 
-   if (eventmode == TEXT2_MODE || eventmode == TEXT3_MODE) {
+   if (eventmode == TEXT_MODE || eventmode == ETEXT_MODE) {
       settext = *((labelptr *)EDITPART);
       setfontencoding(w, value, settext);
       charreport(settext);
@@ -3071,7 +3222,7 @@ Boolean dospecial()
 
 void addtotext(xcWidget w, pointertype value, caddr_t nulldata)
 {
-   if (eventmode != TEXT2_MODE && eventmode != TEXT3_MODE) return;
+   if (eventmode != TEXT_MODE && eventmode != ETEXT_MODE) return;
    if (value == (pointertype)SPECIAL)
       dospecial();
    else
