@@ -284,14 +284,30 @@ typedef struct {
 /* Event mode definitions (state of drawing area)			*/
 /*----------------------------------------------------------------------*/
 
-enum editmode {NORMAL_MODE = 0, UNDO_MODE,
-PENDING_MODE, PRESS_MODE, DELETE_MODE, COPY_MODE, COPY2_MODE, ROTATE_MODE,
-PUSH_MODE, EDIT_MODE, SELAREA_MODE, SELAREA2_MODE, DESEL_MODE, PAN_MODE,
-CATPAN_MODE, CATALOG_MODE, CATTEXT_MODE, FONTCAT_MODE, FONTCAT2_MODE,
-TEXT1_MODE, TEXT2_MODE, TEXT3_MODE, WIRE_MODE, POLY_MODE, BOX0_MODE,
-BOX_MODE, EBOX_MODE, ARC0_MODE, ARC_MODE, EARC_MODE, SPLINE0_MODE,
-SPLINE_MODE, ESPLINE_MODE, EPATH_MODE, LPARAM_MODE, IPARAM_MODE,
-ULPARAM_MODE, UIPARAM_MODE, CONNECT_MODE, ASSOC_MODE, RESCALE_MODE};
+enum editmode {
+  NORMAL_MODE = 0,	/* On the drawing page, none of the situations below */
+  UNDO_MODE,		/* In the process of an undo/redo operation */
+  MOVE_MODE,		/* In the process of moving elements */
+  COPY_MODE,		/* In the process of copying elements */
+  PAN_MODE,		/* In the process of panning to follow the cursor */
+  SELAREA_MODE,		/* Area selection box */
+  RESCALE_MODE,		/* Interactive element rescaling box */
+  CATALOG_MODE,		/* On a library page, library directory, or page directory */
+  CATTEXT_MODE,		/* Editing an existing object name in the library */
+  FONTCAT_MODE,		/* Accessing the font character page from TEXT2_MODE */
+  EFONTCAT_MODE,	/* Accessing the font character page from TEXT3_MODE */
+  TEXT_MODE,		/* Creating a new label */
+  WIRE_MODE,		/* Creating a new polygon (wire) */
+  BOX_MODE,		/* Creating a new box */
+  ARC_MODE,		/* Creating a new arc */
+  SPLINE_MODE,		/* Creating a new spline */
+  ETEXT_MODE,		/* Editing an exiting label */
+  EPOLY_MODE,		/* Editing an existing polygon */
+  EARC_MODE,		/* Editing an existing arc */
+  ESPLINE_MODE,		/* Editing an existing spline */
+  EPATH_MODE,		/* Editing an existing path */
+  ASSOC_MODE		/* Choosing an associated schematic or symbol */
+};
 
 /*----------------------------------------------------------------------*/
 /* File loading modes							*/
@@ -362,7 +378,6 @@ enum loadmodes {IMPORT = 1, PSBKGROUND, SCRIPT, RECOVER,
 #define DEC_INCH	0
 #define FRAC_INCH	1
 #define CM		2
-#define INTERNAL	3
 
 /*----------------------------------------------------------------------*/
 /* Library types							*/
@@ -463,7 +478,7 @@ enum loadmodes {IMPORT = 1, PSBKGROUND, SCRIPT, RECOVER,
 /* Cursor definitions 							*/
 /*----------------------------------------------------------------------*/
 
-#define NUM_CURSORS	10
+#define NUM_CURSORS	11
 
 #define ARROW 		appcursors[0]
 #define CROSS 		appcursors[1]
@@ -475,6 +490,9 @@ enum loadmodes {IMPORT = 1, PSBKGROUND, SCRIPT, RECOVER,
 #define CIRCLE		appcursors[7]
 #define QUESTION	appcursors[8]
 #define WAITFOR		appcursors[9]
+#define HAND		appcursors[10]
+
+#define DEFAULTCURSOR	(*areastruct.defaultcursor)
 
 /*----------------------------------------------------------------------*/
 /* selection-mechanism structures					*/
@@ -484,7 +502,7 @@ typedef XPoint* pointlist;
 typedef XfPoint* fpointlist;
 
 /*----------------------------------------------------------------------*/
-/* Allowed parameterization types ("which" record in oparam structure)	*/
+/* Allowed parameterization types					*/
 /*----------------------------------------------------------------------*/
 
 #define P_NUMERIC	0	/* uncommitted numeric parameter */
@@ -988,6 +1006,9 @@ typedef struct _alias {
 #define xcRemoveTimeOut			XtRemoveTimeOut
 #define xcTimeOutProc			XtTimerCallbackProc
 #define xcIntervalId			XtIntervalId
+#ifndef ClientData
+   #define ClientData			XtPointer
+#endif
 
 #endif
 
@@ -1168,32 +1189,33 @@ enum {
    XCF_View /* 50 */,		XCF_Zoom_In /* 51 */,
    XCF_Zoom_Out /* 52 */,	XCF_Pan /* 53 */,
    XCF_Double_Snap /* 54 */,	XCF_Halve_Snap /* 55 */,
-   XCF_Pan_Left /* 56 */,	XCF_Pan_Right /* 57 */,
-   XCF_Pan_Up /* 58 */,		XCF_Pan_Down /* 59 */,
-   XCF_Write /* 60 */,		XCF_Rotate /* 61 */,
-   XCF_Flip_X /* 62 */,		XCF_Flip_Y /* 63 */,
-   XCF_Snap /* 64 */,		XCF_SnapTo /* 65 */,
-   XCF_Pop /* 66 */,		XCF_Push /* 67 */,
-   XCF_Delete /* 68 */,		XCF_Select /* 69 */,
-   XCF_Box /* 70 */,		XCF_Arc /* 71 */,
-   XCF_Text /* 72 */,		XCF_Exchange /* 73 */,
-   XCF_Copy /* 74 */,		XCF_Move /* 75 */,
-   XCF_Join /* 76 */,		XCF_Unjoin /* 77 */,
-   XCF_Spline /* 78 */,		XCF_Edit /* 79 */,
-   XCF_Undo /* 80 */,		XCF_Redo /* 81 */,
-   XCF_Select_Save /* 82 */,	XCF_Unselect /* 83 */,
-   XCF_Dashed /* 84 */,		XCF_Dotted /* 85 */,
-   XCF_Solid /* 86 */,		XCF_Prompt /* 87 */,
-   XCF_Dot /* 88 */,		XCF_Wire /* 89 */,
-   XCF_Start /* 90 */,		XCF_Finish /* 91 */,
-   XCF_Cancel /* 92 */,		XCF_Nothing /* 93 */,
-   XCF_Exit /* 94 */,		XCF_Netlist /* 95 */,
-   XCF_Swap /* 96 */,		XCF_Pin_Label /* 97 */,
-   XCF_Pin_Global /* 98 */,	XCF_Info_Label /* 99 */,
-   XCF_Graphic	/* 100 */,	XCF_Rescale /* 101 */,
-   XCF_Connectivity /* 102 */,	XCF_Sim /* 103 */,
-   XCF_SPICE /* 104 */,		XCF_PCB /* 105 */,
-   XCF_SPICEflat /* 106 */,	XCF_Text_Delete_Param,	/* 107 */
+   XCF_Write /* 56 */,		XCF_Rotate /* 57 */,
+   XCF_Flip_X /* 58 */,		XCF_Flip_Y /* 59 */,
+   XCF_Snap /* 60 */,		XCF_SnapTo /* 61 */,
+   XCF_Pop /* 62 */,		XCF_Push /* 63 */,
+   XCF_Delete /* 64 */,		XCF_Select /* 65 */,
+   XCF_Box /* 66 */,		XCF_Arc /* 67 */,
+   XCF_Text /* 68 */,		XCF_Exchange /* 69 */,
+   XCF_Copy /* 70 */,		XCF_Move /* 71 */,
+   XCF_Join /* 72 */,		XCF_Unjoin /* 73 */,
+   XCF_Spline /* 74 */,		XCF_Edit /* 75 */,
+   XCF_Undo /* 76 */,		XCF_Redo /* 77 */,
+   XCF_Select_Save /* 78 */,	XCF_Unselect /* 79 */,
+   XCF_Dashed /* 80 */,		XCF_Dotted /* 81 */,
+   XCF_Solid /* 82 */,		XCF_Prompt /* 83 */,
+   XCF_Dot /* 84 */,		XCF_Wire /* 85 */,
+   XCF_Cancel /* 86 */,		XCF_Nothing /* 87 */,
+   XCF_Exit /* 88 */,		XCF_Netlist /* 89 */,
+   XCF_Swap /* 90 */,		XCF_Pin_Label /* 91 */,
+   XCF_Pin_Global /* 92 */,	XCF_Info_Label /* 93 */,
+   XCF_Graphic	/* 94 */,	XCF_Rescale /* 95 */,
+   XCF_SelectBox /* 96 */,	XCF_Connectivity /* 97 */,
+   XCF_Continue_Copy /* 98 */,	XCF_Continue_Element /* 99 */,
+   XCF_Finish_Copy /* 100 */,	XCF_Finish_Element /* 101 */,
+   XCF_Finish /* 102 */,	XCF_Cancel_Last /* 103 */,
+   XCF_Sim /* 104 */,		XCF_SPICE /* 105 */,
+   XCF_PCB /* 106 */,		XCF_SPICEflat /* 107 */,
+   XCF_Text_Delete_Param /* 108 */,
    NUM_FUNCTIONS
 };
 
@@ -1290,6 +1312,7 @@ typedef struct {
    short	editcycle;
    short	editpart, editsubpart;
    char		*lastbackground;
+   Cursor	*defaultcursor;
 } Clientdata;
 
 /* Record for undo function */ 
@@ -1326,9 +1349,9 @@ typedef struct {
 typedef struct {
    char		*tempfile;
    char		*tempdir;
-   short	do_replace;
    xcIntervalId	timeout_id;
    int		save_interval;
+   short	do_replace;
    u_short	new_changes;
    short	numlibs;
    short	pages;
@@ -1365,6 +1388,8 @@ typedef struct _menustruct {
 /* Structure for calling routines from the Toolbar icons		*/
 /*----------------------------------------------------------------------*/
 
+typedef struct _toolbarstruct *toolbarptr;
+
 typedef struct _toolbarstruct {
 	char *name;
 	char **icon_data;
